{%- extends "layout.html" %}
{% set title = _('Search') %}
{%- block scripts %}
    {{ super() }}
{%- endblock %}
{% block extrahead %}
    <meta name="robots" content="noindex" />
    {{ super() }}
{% endblock %}
{% block body %}
{% endblock %}
