<input id="input-switch-toc" type="checkbox">
<label id="show-repotoc" for="input-switch-toc">
All content
</label>
<label id="show-toc" for="input-switch-toc">
Content on this topic
</label>
<div class="repotoc-tree">
  {{ repotoc_tree }}
</div>
<div class="toc-tree">
  {{ toc_tree }}
</div>
