@use "variable";

.sig-object {
    font-family: monospace;
    font-size: 1rem;
}

.sig-prename, .sig-name {
    color: var(--accent-color-code);
}

.sig-name, em.property {
    font-weight: 600;
}

em.sig-param, em.property {
    font-style: normal;
}

.sig-object {
    transition: ease background-color .25s;
    background-color: rgba(125, 125, 125, 0);
    padding: .25em .5rem;
    border-radius: 1rem;
}

.sig-object:hover {
    background-color: rgba(125, 125, 125, 0.1);
}

.code-block-caption {
    text-align: center;
    padding-bottom: .25em;
}


.highlight {
    background-color: var(--bg-color3) !important;
}

// TODO Port to third-party themes
.code-shell {
    background-color: var(--bg-color3);
    border-radius: variable.$border-radius;
    padding: .75em;
    margin-bottom: .5em;
    user-select: text;
    overflow-x: auto;

    .highlight {
        padding: 0;
        white-space: collapse;
        margin: 0;
        background: none !important;
        overflow: visible;
    }

    .float-left {
        float: left;
        clear: left;
        display: inline-block;
        margin-right: .25em;
        color: var(--accent-color);
    }

    pre {
        margin: 0;
    }

    .no-select {
        opacity: 0.8;
        user-select: none;
    }

    & > div {
        padding-bottom: .25em;
    }
}

