@use "sass:list";
$links: (
    "adi":     #0067b9 "adi",
    "ez":      #2eb251 "ez",
    "git":     #f05133 "git",
    "link":    #666666 "link",
    "dokuwiki":#666666 "dokuwiki",
    "download":#666666 "download",
    "doxygen": #659ad2 "doxygen",
);

@mixin default-link($color, $icon-name) {
    --color-link-title: #{$color};
    --color-link-title-background: #{rgba($color, 0.2)};

    --icon-link-default: var(--icon-#{$icon-name});
}

@mixin links {
    @each $name, $values in $links {
        --color-link-title--#{$name}: #{list.nth($values, 1)};
        --color-link-title-background--#{$name}: #{rgba(
                list.nth($values, 1),
                0.2
            )};
    }
}
