$hmc-color-orange-500: #ae4410;
$hmc-color-red-700: #c81a28;
$hmc-color-blue-700: #0067b9;
$hmc-color-blue-300: #92befc;
$hmc-color-purple-700: #8e3dc7;
$hmc-color-purple-500: #b16ee0;
$hmc-color-purple-300: #d4adf0;
$hmc-radius-200: 0.5rem;
$hmc-radius-300: 0.75rem;
$hmc-radius-400: 1rem;

$bg-light1: #f8f8f8;
$bg-light2: #f0f1f3;
$bg-light3: #fefefe;
$bg-dark1: #1a1a1a;
$bg-dark2: #111;
$bg-dark3: #202020;
$bg-banner-dark: #0088ff;
$bg-banner-light: #00305b;
$text-color-dark1: #e5e5e5;
$text-color-dark2: #ccc;
$text-color-dark3: #aaa;
$text-color-light1: #212836;
$text-color-light2: #555;
$text-color-light3: #888;
$text-color-banner-light: #fff;
$text-color-banner-dark: #000;
$box-shadow-left-sidebar: .3rem 0 .3rem rgba(0,0,0,.1), 0 0 .075rem rgba(0,0,0,.15);
$box-shadow-right-sidebar: -0.3rem 0 .3rem rgba(0,0,0,.1), 0 0 .075rem rgba(0,0,0,.15);
$box-shadow-raised: 0 .2rem .5rem rgba(0,0,0,.05),0 0 .0625rem rgba(0,0,0,.1);
$border-panel1: 1px solid rgba(125, 125, 125, 0.8);
$border-panel2: 1px solid rgba(125, 125, 125, 0.5);
$border-th: 1px solid rgba(125, 125, 125, 0.5);
$border-td: 1px solid rgba(125, 125, 125, 0.25);
$accent-color-light: $hmc-color-blue-700;
$accent-color-dark: $hmc-color-blue-300;
$accent-color-code-light: $hmc-color-purple-700;
$accent-color-code-dark: $hmc-color-purple-300;
$border-radius: $hmc-radius-200;
$border-radius-extra: $hmc-radius-400;

$screen-3: 105em;
$screen-2: 80em;
$screen-1: 65em;
$screen-0: 40em;

