/*
 * Styles for directives and roles for third-party themes.
 * This is an exceptional asset that gets inject to the
 * html if the theme is from a third party.
 * The contents should be kept to a minimum, if necessary,
 * the user can include custom styles on his side.
 */

iframe {
    border: none;
}

video {
    width: 100%;
}

.bold {
    font-weight: bold;
}

.red {
    color: red;
}

.green {
    color: green;
}

.collapsible {
    border: 1px solid var(--color-table-border);
    border-radius: .25em;
    margin-bottom: -1px;
}

.collapsible_input {
    display: none;
}

.collapsible_content {
    border-top: 1px solid var(--color-table-border);
    overflow: hidden;
    height: 0;
    padding: 0 .75em !important;
    transition: ease opacity .25s;
    opacity: 0;
}

.collapsible label {
    width: 100%;
    padding: 0.75em 1em 0.75em .75em;
    user-select: none;
    box-sizing: border-box;
    cursor: pointer;
    display: flex;
    align-items: center;
    justify-content: space-between;
    font-weight: bold;
    color: var(--color-toc-item-text);
}

.collapsible label .icon {
    border: solid var(--color-toc-item-text);
    border-width: 0 2px 2px 0;
    display: block;
    transition: transform ease .125s, margin-top ease .125s;
    width: .6em;
    height: .6em;
    transform: rotate(-45deg);
}

.collapsible label p {
    display: inline-block;
    margin: 0;
}

.collapsible label, .collapsible div {
    transition: box-shadow ease .25s;
}

.collapsible label:hover ~ div, .collapsible label:hover {
    box-shadow: 0 .2rem .5rem rgba(0,0,0,.05),0 0 .0625rem rgba(0,0,0,.1);
}

.collapsible_input:checked ~ label .icon {
    transform: rotate(45deg);
    margin-top: -.5em;
}

.collapsible_content {
    overflow: auto;
}

.collapsible_input:checked ~ .collapsible_content {
    height: 100%;
    opacity: 1;
}

.esd-warning {
    display: flex;
    border-radius: .5em;
    align-items: center;
    padding: .25rem .5em !important;
    background: yellow;
    font-size: .9em;
    color: #000 !important;
    text-align: justify;
}

.esd-warning .icon {
    width: 7rem;
    height: 7rem;
    margin-right: .5rem;
    flex-shrink: 0;
    background: url(esd-warning.svg);
    background-size: contain;
}

#hdl-parameters table p,
#hdl-interfaces table p,
#hdl-regmap table p {
    white-space: break-spaces;
}
