@font-face {
    font-family: "Barlow";
    font-display: fallback;
    src: url('fonts/barlow/Barlow-Medium.woff2') format('woff2');
    font-weight: 500;
    font-style: normal;
}

@font-face {
    font-family: "Barlow";
    font-display: fallback;
    src: url('fonts/barlow/Barlow-SemiBold.woff2') format('woff2');
    font-weight: 600;
    font-style: normal;
}

@font-face {
    font-family: "Inter";
    font-display: fallback;
    src: url('fonts/inter/Inter-Regular.woff2') format('woff2');
    font-weight: normal;
    font-style: normal;
}

@font-face {
    font-family: "Inter";
    font-display: fallback;
    src: url('fonts/inter/Inter-SemiBold.woff2') format('woff2');
    font-weight: 600;
    font-style: normal;
}

body {
    font-family: Inter, Arial, sans-serif;
}

.body :is(h1, h2, h3, h4, h5, h6) {
    font-family: Barlow, Arial, sans-serif;
    font-weight: 500;
    margin: 0;
    padding: 1em 0 .4em 0;
}

.body h1 {
    font-size: 1.7em;
}

.body h2 {
    font-size: 1.6em;
}

.body h3 {
    font-size: 1.4em;
}

.body h4 {
    font-size: 1.3em;
}

.body h5 {
    font-size: 1.25em;
}

.body h6 {
    font-size: 1.2em;
}

.sphinxsidebar .caption-text {
    font-family: Barlow, Arial, sans-serif;
    font-weight: 600;
    font-size: 1.1em;
}

.bold {
    font-weight: 600;
}
